"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.trace = exports.sdk = exports.loggingMiddleware = exports.loggerDebugger = exports.logger = exports.DiagLogLevel = void 0;
const api_1 = require("@opentelemetry/api");
Object.defineProperty(exports, "trace", { enumerable: true, get: function () { return api_1.trace; } });
const api_2 = require("@opentelemetry/api");
Object.defineProperty(exports, "DiagLogLevel", { enumerable: true, get: function () { return api_2.DiagLogLevel; } });
const api_logs_1 = require("@opentelemetry/api-logs");
const auto_instrumentations_node_1 = require("@opentelemetry/auto-instrumentations-node");
const exporter_logs_otlp_http_1 = require("@opentelemetry/exporter-logs-otlp-http");
const exporter_metrics_otlp_http_1 = require("@opentelemetry/exporter-metrics-otlp-http");
const exporter_trace_otlp_http_1 = require("@opentelemetry/exporter-trace-otlp-http");
const resources_1 = require("@opentelemetry/resources");
const sdk_logs_1 = require("@opentelemetry/sdk-logs");
const sdk_metrics_1 = require("@opentelemetry/sdk-metrics");
const sdk_node_1 = require("@opentelemetry/sdk-node");
const semantic_conventions_1 = require("@opentelemetry/semantic-conventions");
const dotenv_1 = require("dotenv");
const express_1 = __importDefault(require("express"));
(0, dotenv_1.config)();
const expHistogramView = new sdk_metrics_1.View({
    aggregation: new sdk_metrics_1.ExponentialHistogramAggregation(),
    // Note, the instrumentName is the same as the name that has been passed for
    // the Meter#createHistogram function for exponentialHistogram.
    instrumentName: 'test_exponential_histogram',
});
//
// For troubleshooting, set the log level to DiagLogLevel.DEBUG
const loggerDebugger = (LogLevel) => api_2.diag.setLogger(new api_2.DiagConsoleLogger(), LogLevel);
exports.loggerDebugger = loggerDebugger;
const env = {
    service: {
        name: process.env.SERVER_HOSTNAME || 'undefined service',
        version: process.env.npm_package_version || 'undefined service version',
    },
    collector: {
        url: process.env.OPENTELEMETRY_COLLECTOR_URL || 'http://otelcol:4318',
    },
};
const sdk = new sdk_node_1.NodeSDK({
    resource: new resources_1.Resource({
        [semantic_conventions_1.SemanticResourceAttributes.SERVICE_NAME]: env.service.name,
        [semantic_conventions_1.SemanticResourceAttributes.SERVICE_VERSION]: env.service.version,
    }),
    views: [expHistogramView],
    traceExporter: new exporter_trace_otlp_http_1.OTLPTraceExporter({
        url: `${env.collector.url}/v1/traces`,
    }),
    metricReader: new sdk_metrics_1.PeriodicExportingMetricReader({
        exporter: new exporter_metrics_otlp_http_1.OTLPMetricExporter({
            url: `${env.collector.url}/v1/metrics`,
        }),
        exportIntervalMillis: 1000,
    }),
    logRecordProcessor: new sdk_logs_1.BatchLogRecordProcessor(new exporter_logs_otlp_http_1.OTLPLogExporter({
        url: `${env.collector.url}/v1/logs`,
    })),
    instrumentations: [(0, auto_instrumentations_node_1.getNodeAutoInstrumentations)()],
});
exports.sdk = sdk;
const meter = api_1.metrics.getMeter(env.service.name);
const requestCounter = meter.createCounter('requests', {
    description: 'Example of a Counter',
});
const upDownCounter = meter.createUpDownCounter('test_up_down_counter', {
    description: 'Example of a UpDownCounter',
});
const histogram = meter.createHistogram('test_histogram', {
    description: 'Example of a Histogram',
});
const exponentialHistogram = meter.createHistogram('test_exponential_histogram', {
    description: 'Example of an ExponentialHistogram',
});
const attributes = { pid: process.pid, environment: 'staging' };
setInterval(() => {
    requestCounter.add(1, attributes);
    upDownCounter.add(Math.random() > 0.5 ? 1 : -1, attributes);
    histogram.record(Math.random(), attributes);
    exponentialHistogram.record(Math.random(), attributes);
}, 1000);
const app = (0, express_1.default)();
/**
 * Open Telemtry Logging middleware
 */
const loggingMiddleware = app.use((req, res, next) => {
    try {
        const activeSpan = api_1.trace.getActiveSpan();
        const logger = api_logs_1.logs.getLogger(env.service.name);
        if (activeSpan !== undefined && logger !== undefined) {
            const temp = res.json;
            const log = ({ severityNumber, severityText, context, message }) => {
                const obj = {
                    severityNumber,
                    severityText,
                    body: JSON.stringify({ context, message }),
                };
                console.log('--- LOG ---', obj);
                logger.emit(obj);
            };
            res.json = function (...args) {
                log({
                    severityNumber: api_logs_1.SeverityNumber.INFO,
                    severityText: 'INFO',
                    context: 'express request log',
                    message: {
                        request: {
                            headers: req.headers,
                            body: req.body,
                            query: req.query,
                            params: req.params,
                        },
                        response: {
                            body: args['0'],
                        },
                    },
                });
                return temp.apply(this, args);
            };
        }
    }
    catch (e) {
        console.log('--- ERR :( ---', e);
    }
    next();
});
exports.loggingMiddleware = loggingMiddleware;
const logger = api_logs_1.logs.getLogger(env.service.name);
exports.logger = logger;
